/**
ATLAS model for GAMA 1.7 - 07/02/2017
Authors : Hugo Thierry, Claude Monteil, and Aude Vialatte (UMR DYNAFOR, Toulouse, France) - Benoit Gaudou (UMR IRIT, Toulouse, France)
If you publish results thanks to the ATLAS model, please cite ATLAS model in your publications.
ATLAS model is free software under GNU GPL v3 license; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free Software Foundation; 
  either version 3 of the License, or (at your option) any later version.
ATLAS model is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU General Public License for more details: https://www.gnu.org/licenses/gpl-3.0.en.html 
 */
model ATLASmodel

global // All interface parameters are declared in the global in GAMA
{

// Attributes
	string PARAMETERS_FOLDER_PATH <- '../includes/' const:true;  // path of the folder containing all parameters files
	string PARAMETERS_FILE_NAME <- '_Parameters.csv' const:true; // name of the file containing ATLAS parameters such as date files
	file LANDSCAPE_GIS_FILE <- shape_file(PARAMETERS_FOLDER_PATH+'Landscape.shp') const:true; // GIS File
	
	file PARAMETERS_FILE <- csv_file(PARAMETERS_FOLDER_PATH+PARAMETERS_FILE_NAME) const:true; 
		//Default set of parameters to initialize the simulation
	file climateCSV;			// Wind and Temperature File
	file rainCSV;				// Rain File
	file rotationsCSV;			// Rotations File
	file landcoverCSV;			// Landcovers File
	file landuseCSV;			// Landuses File	
	file cropStagesCSV;			// Crop Stages File											

	bool hasHourlyWeatherData;	// Is climate data at an hourly timestep ?
	bool hasInputRotations; 	// Are rotations already assigned to patches in the GIS data ?
	
	int timestepHours; 			// Hour many hours does one timestep represent in the simulation	
	int initialDay;				// Initial day of the simulation
	int initialMonth;
	int julianDay;				// Day of the year
	int daysElapsed;			// Number of days since the simulation started
	int hour;					// Hour of the day	
	int year;					// Year of the simulation
	
	Landscape myLandscape; 		//Create the landscape agent
	
	geometry shape <- envelope(LANDSCAPE_GIS_FILE);	// Limit the model to the GIS boundaries
	
// Examples of possible outputs to follow through the simulation
// Follow the evolution of the area assigned to crops
	float areaWheat;
	float areaRapeseed;
	float areaCorn;
	float areaTemporarypasture;
	float areaSorghum;
	float areaSunflower;

	// And the different crop stages of Wheat
	float areaWheatSowed;
	float areaWheatEmerg;
	float areaWheatBoot;
	float areaWheatHead;
	float areaWheatFlow;
	float areaWheatMaturity;
	float areaWheatHarvest;
	reflex update_values_graphs when: (hour = 0)
	{
	// Purpose:  Updates the total area assigned to crops and the area assigned to wheat crop stages as an example
	// InOut : All the above areas
		areaWheat <- 0.0;
		areaRapeseed <- 0.0;
		areaCorn <- 0.0;
		areaTemporarypasture <- 0.0;
		areaSorghum <- 0.0;
		areaSunflower <- 0.0;
		ask myLandscape.patchesList
		{
			if currentLandcover.name = "CoverWheat"
			{
				areaWheat <- areaWheat + size;
			}

			if currentLandcover.name = "CoverRapeseed"
			{
				areaRapeseed <- areaRapeseed + size;
			}

			if currentLandcover.name = "CoverCorn"
			{
				areaCorn <- areaCorn + size;
			}

			if currentLandcover.name = "CoverTemporarypasture"
			{
				areaTemporarypasture <- areaTemporarypasture + size;
			}

			if currentLandcover.name = "CoverSorghum"
			{
				areaSorghum <- areaSorghum + size;
			}

			if currentLandcover.name = "CoverSunflower"
			{
				areaSunflower <- areaSunflower + size;
			}

		}

		areaWheatSowed <- 0.0;
		areaWheatEmerg <- 0.0;
		areaWheatBoot <- 0.0;
		areaWheatHead <- 0.0;
		areaWheatFlow <- 0.0;
		areaWheatMaturity <- 0.0;
		areaWheatHarvest <- 0.0;
		ask myLandscape.patchesList
		{
			if plannedLanduse.name = "Wheat"
			{
				if phenologicalStage = "Sowed"
				{
					areaWheatSowed <- areaWheatSowed + size;
				}

			}

			if currentLandcover.name = "CoverWheat"
			{
				if phenologicalStage = "Emergence"
				{
					areaWheatEmerg <- areaWheatEmerg + size;
				}

				if phenologicalStage = "Boot"
				{
					areaWheatBoot <- areaWheatBoot + size;
				}

				if phenologicalStage = "Head"
				{
					areaWheatHead <- areaWheatHead + size;
				}

				if phenologicalStage = "Flowering"
				{
					areaWheatFlow <- areaWheatFlow + size;
				}

				if phenologicalStage = "Maturity"
				{
					areaWheatMaturity <- areaWheatMaturity + size;
				}

				if phenologicalStage = "Harvestable"
				{
					areaWheatHarvest <- areaWheatHarvest + size;
				}

			}

		}

	}
	
	// Methods
	init{
	// Purpose : initialize global variables
		matrix fichiersData <- matrix(PARAMETERS_FILE);		// Launcher file with all includes listed
		climateCSV <- csv_file(PARAMETERS_FOLDER_PATH+string(fichiersData[1,1]),false);	
			// All input files are initialized from the above launcher CSV
		rotationsCSV <- csv_file(PARAMETERS_FOLDER_PATH+string(fichiersData[1,2]));	
		landcoverCSV <- csv_file(PARAMETERS_FOLDER_PATH+string(fichiersData[1,3]));	
		landuseCSV <- csv_file(PARAMETERS_FOLDER_PATH+string(fichiersData[1,4]));		
		cropStagesCSV <- csv_file(PARAMETERS_FOLDER_PATH+string(fichiersData[1,5]),false);	
		rainCSV <- csv_file(PARAMETERS_FOLDER_PATH+string(fichiersData[1,6]),false);			
		initialDay<-int(fichiersData[1,7]);						
		initialMonth<-int(fichiersData[1,8]);
		hasHourlyWeatherData<-bool(fichiersData[1,9]);
		if hasHourlyWeatherData {
			timestepHours<-int(fichiersData[1,10]);
		}
		else{
			timestepHours<-8;
		}
		hasInputRotations<-bool(fichiersData[1,11]);	
		
		list<int> daysPerMonth <- [0,31,59,90,120,151,181,212,243,273,304,334];	// Transform into Julian Date
		julianDay<-daysPerMonth[initialMonth-1]+initialDay;
	

		hour <-24-timestepHours;// Start just before the first day
		daysElapsed <--1;
		year <-0;				// Start at year = 0;
		
		create Landscape number:1 returns: createdLandscape; // Creates one landscape 
	}
			
	reflex update_time{
	// Purpose: Update the year, day and hour of the simulation
	// InOut :self.year, self.day, self.hour, self.numberOfDays			
		hour <- hour +timestepHours;			
		if hour = 24 {hour <- 0; julianDay <- julianDay + 1; daysElapsed <- daysElapsed +1;}
		if julianDay = 366 {year <- year+1;julianDay <- 1;}	
	}

}

species Landscape
{
// Attributes
	list<Landuse> landusesList <- []; // List of the landuses
	list<Landcover> landcoversList <- []; // List of the landcovers
	list<Rotation> rotationsList <- []; // List of the rotations
	list<Patch> patchesList <- []; // List of the patches
	Climate climate; // Climate applied to the landscape

	// Methods
	init
	{
	// Purpose: create the components of a Landscape from the global CSV and SHP files
	// out: self.landusesList, self.rotationsList, self.patchesList, self.climate		
		world.myLandscape <- self; // createdLandscape is a list of one entity
		create Climate number: 1 returns: createdClimate; // Create the climate entity
		climate <- any(createdClimate);
		do create_landuses(); // Create the landuses
		do create_landcovers(); // Create the landcovers
		do create_rotations(); // Create the rotations
		do create_patches(); // Create the patches composing the landscape
	}

	reflex timestep_update
	{
	// Purpose : Define the actions done at each timestep	
		ask climate
		{
			do update_wind; // Update the wind
			do update_temperature; // Update temperature
		}

	}

	reflex daily_update when: hour = 0
	{
	// Purpose : Define the actions done at a daily timestep (at midnight)
		ask climate
		{
			do update_rain; // Update rain
		}

		do update_landcovers; // Update the landcovers
	}

	action create_landuses
	{
	// Purpose : Create the list of landuses used by the landscape
	// In: landuseCSV, cropStagesCSV 
	// Out: landusesList
		matrix landuseData <- matrix(landuseCSV); //import landuses
		matrix coverStagesData <- matrix(cropStagesCSV); // import cover stages
		loop x from: 0 to: ((landuseData column_at (0)) count (each != '') - 1)
		{
		// Each loop creates a landuse and defines its attributes 
			create Landuse number: 1 returns: newCover
			{
				name <- string(landuseData[0, x]); // name
				abbreviation <- string(landuseData[1, x]); // abbreviation
				isDynamicThroughTime <- bool(landuseData[2, x]); // is the landuse temporally dynamic ?
				if isDynamicThroughTime
				{
					isPartOfRotation <- bool(landuseData[3, x]); // is it part of crop rotations ?
					isPhenologicallyDetailed <- bool(landuseData[4, x]); // is it phenologically detailed ?
					isAnnual <- bool(landuseData[5, x]); // is it an annual crop ?
					if isAnnual or isPartOfRotation
					{
						startSowingDate <- int(landuseData[6, x]); // Date when sowing  starts
						maximumHarvestDate <- int(landuseData[7, x]); // Date when harvest ends
					}

					if isPhenologicallyDetailed
					{
						baseTemperature <- int(landuseData[8, x]); // Base temperature to calculate cumulated degree days
						if isPartOfRotation or isAnnual
						{
							cropSowingDelay <- int(coverStagesData[1, x]); // Sowing window (in days)
							cropHarvestDelay <- int(coverStagesData[2, x]); // Harvest window (in days)
							int y <- 3;
							loop while: y <= (((coverStagesData row_at (x)) count (each != '') - 1))
							{
							// phenologicalStages <- phenologicalStages + [coverStagesData[y,x]];	 // Phenological stage of the crop					
								add coverStagesData[y, x] to: phenologicalStages;
								// GDDThresholds <- GDDThresholds + coverStagesData[y+1,x];		// Growing degree days threshold to reach the stage
								add coverStagesData[y + 1, x] to: GDDThresholds;
								// phenologicalHeight<-phenologicalHeight + coverStagesData[y+2,x];	 // Height of the stage in the model
								add coverStagesData[y + 2, x] to: phenologicalHeight;
								y <- y + 3;
							}

							nbCropStages <- length(phenologicalStages); // Count the number of phenological stages
						}

					}

					isClustered <- bool(landuseData[9, x]); // Is the crop spatially clustered ?
				}

				index <- x - 1;
			}

			landusesList <- landusesList + newCover; // Update the list of landuses with the created one
		}

	}

	action create_landcovers
	{
	// Purpose : Create the list of landcovers used by the landscape
	// In: landcoverCSV 
	// Out: landcoversList		
		matrix landcoverMatrix <- matrix(landcoverCSV); // Input the landcovers
		loop x from: 0 to: ((landcoverMatrix column_at (0)) count (each != '') - 1)
		{
		// Each loop creates a landcover 
			create Landcover number: 1 returns: newCover
			{
				name <- string(landcoverMatrix[0, x]); // name
				abbreviation <- string(landcoverMatrix[1, x]); // abbreviation
				color <- rgb([int(landcoverMatrix[2, x]), int(landcoverMatrix[3, x]), int(landcoverMatrix[4, x])]); // RGB color
				baseHeight <- int(landcoverMatrix[5, x]); // Base height of the crop when not phenologically detailed
			}

			landcoversList <- landcoversList + newCover; // Update the list of landcovers with the created one
		}

	}

	action create_rotations
	{
	// Purpose : Create the list of Rotations used by the landscape
	// In: rotationsCSV
	// Out:rotationsList
		matrix rotationsMatrix <- matrix(rotationsCSV);
		loop y from: 0 to: (rotationsMatrix column_at (0)) count (each != '') - 1
		{
		// Each loop creates a rotation	
			create Rotation number: 1 returns: newRotation
			{
				name <- (string(rotationsMatrix[0, y])); // Name
				referenceArea <- float(rotationsMatrix[1, y]); // Reference area to assign to the rotation
				index <- y;
				loop x from: 2 to: (rotationsMatrix row_at (y)) count (each != "") - 1
				{ // Go through the different landuses
					Landuse successionCrop <- myself.landusesList first_with (each.name = (rotationsMatrix[x, y])); // Identify the landuse
					succession <- succession + successionCrop; // Add it to the succession of landuses
					phenologyDates <- phenologyDates + (int(successionCrop.startSowingDate) - 1); // Add the date of sowing of the landuse to the list of dates 
					phenologyDates <- phenologyDates + successionCrop.maximumHarvestDate; // Add the date of harvest of the landuse to the list of dates
					if x = 2
					{
						firstDate <- int(successionCrop.startSowingDate); // If first landuse, the date is the first date of the list
					}

				}

				loop z from: 1 to: phenologyDates count (each != '') - 1
				{ // Transform the julian dates to a number of days elapsed
					if phenologyDates[z - 1] > phenologyDates[z]
					{
						loop x from: z to: phenologyDates count (each != '') - 1
						{
							phenologyDates[x] <- phenologyDates[x] + 365;
						}

					}

				}

				duration <- int(ceil(((int(phenologyDates[phenologyDates count (each != '') - 1])) - (int(firstDate))) / 365));
				// Count the duration of the rotation
			}

			rotationsList <- rotationsList + newRotation; // Add the rotation to the list of rotations
		}

	}

	action create_patches
	{
	// Purpose : Create the list of Patches used by the landscape
	// In: landscapeGIS
	// Out: patchesList	
		if hasInputRotations = false
		{ // If ATLAS needs to assign rotations
			create Patch from: LANDSCAPE_GIS_FILE with: [readLanduse::string(read("Cover")), size::int(read("Area")), iD::int(read("Id"))] returns: patchesCreated
			{ //
			// Here the landuse will be read from the Cover column of the GIS file
			// The size of the patch will be read from the Area column
			// And finally the iD of the patch is read from the Id column
				Landuse sameLanduse <- myself.landusesList first_with (each.name = self.readLanduse);
				currentLandcover <- myself.landcoversList first_with (each.name = "Cover" + sameLanduse.name);
				// Assign the landcover to the patch from the list of landcovers
				plannedLanduse <- sameLanduse; // Assign the landuse from the list of landuses
				GDDCumulated <- 0.0; // Cumulated degrees of the cover of the patch set to 0
				isInitialLanduse <- false; // Default value (crop not available at starting date)
				isInitialPatch <- false; // Default value (patch is not a reference for rotation attribution)
				hasRotationAssigned <- false; // Default value (no rotation assigned yet)			
			}

			patchesList <- patchesCreated; // Update the list of patches with the patch created
			do assign_rotations; // Assign rotations to all crop patches
			do setup_phenology; // Setup the initial state of these patches
		} else
		{ // If ATLAS doesn't need to assign rotations
			create Patch from: LANDSCAPE_GIS_FILE with:
				[readLanduse::string(read("Cover")), size::int(read("Area")), readRotation::string(read("Rotation")), successionIndex::int(read("SucIndex")), iD::int(read("Id"))] 
				returns: patchesCreated
			{
			// Here the landuse will be read from the Cover column of the GIS file
			// The size of the patch will be read from the Area column
			// The rotation from the Rotation column
			// The succession index from the SucIndex column
			// And finally the iD of the patch is read from the Id column					
				Landuse sameLanduse <- myself.landusesList first_with (each.name = self.readLanduse);
				rotation <- myself.rotationsList first_with (each.name = self.readRotation); // Assign the rotation to the patch
				currentLandcover <- myself.landcoversList first_with (each.name = "Cover" + sameLanduse.name);
				// Assign the landcover to the patch from the list of landcovers
				plannedLanduse <- sameLanduse; // Assign the landuse from the list of landuses
				GDDCumulated <- 0.0; // Cumulated degrees of the cover of the patch set to 0
				isInitialLanduse <- false; // Default value (crop not available at starting date)
			}

			patchesList <- patchesCreated; // Update the list of patches with the patch created
			do setup_phenology; // Setup the initial state of these patches			
		}

	}

	action assign_rotations
	{
	// Purpose: assign a rotation to the crop patches
	// In: rotationsList, patchesList  
	// Out: patch.successionIndex, patch.rotation, patch.plannedLanduse  
	// InOut: patch.currentLandcover
		loop i from: 0 to: (rotationsList count (each != '') - 1)
		{ // For each rotation defined, we assign a reference patch
			ask Rotation where (each.index = i)
			{
				ask any(myLandscape.patchesList where (((each.plannedLanduse = any(self.succession))) and (each.hasRotationAssigned = false)))
				{
				// Assign the rotation to a patch with a landuse found in the rotation
					self.rotation <- any(Rotation where (each.index = i)); // Update rotation for the patch
					self.isInitialPatch <- true; // This patch will be a reference for clustering
					self.hasRotationAssigned <- true; // This patch has been assigned a rotation
					do assign_initial_landuse(rotation); // Define a starting point among the rotation		
					plannedLanduse <- myLandscape.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
					// Assign a landuse according to the starting point
				}

			}

		}

		loop i from: 0 to: (rotationsList count (each != '') - 2)
		{ // We then assign for all rotations but the last patches until the reference area is met
			Rotation refRotation <- Rotation(rotationsList[i]);
			ask any(Patch where (each.rotation = refRotation))
			{
				refRotation.areaAssigned <- int(size);
				if rotation.succession count (each.isClustered) > 0
				{ // If one of the landuses of the rotation needs to be clustered
					loop while: (refRotation.areaAssigned < refRotation.referenceArea - (refRotation.referenceArea * 2) / 100)
					{
					// We allow 2% error in the area assigned to the rotation												
					// Neighboring crop patches are assigned when the rotation needs clustering
						Patch Neighbor <- myLandscape.patchesList where ((each.hasRotationAssigned = false) and (each.plannedLanduse.isPartOfRotation) and
						(each.size < ((refRotation.referenceArea + (refRotation.referenceArea * 5) / 100)) - refRotation.areaAssigned)) closest_to self; // {
						ask Neighbor
						{
							rotation <- myself.rotation;
							hasRotationAssigned <- true;
							do assign_initial_landuse(rotation);
							plannedLanduse <- myLandscape.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
							refRotation.areaAssigned <- int(refRotation.areaAssigned + size);
						}

					}

				} else
				{ // If none of the landuses of the rotation need to be clustered
					loop while: (refRotation.areaAssigned < refRotation.referenceArea - (refRotation.referenceArea * 2) / 100)
					{
					// We allow 2% error in the area assigned to the rotation				
					// Any crop patch can be assigned when there is no need for clustering
						ask any(Patch where ((each.hasRotationAssigned = false) and (each.plannedLanduse.isPartOfRotation)))
						{
							rotation <- any(Rotation where (each.index = (i)));
							hasRotationAssigned <- true;
							do assign_initial_landuse(rotation);
							plannedLanduse <- myLandscape.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
							refRotation.areaAssigned <- int(refRotation.areaAssigned + size);
						}

					}

				}

			}

		}

		ask (Patch where ((each.hasRotationAssigned = false) and (each.plannedLanduse.isPartOfRotation)))
		{
		// Finally the last rotation is assigned to the remaining crop patches
			rotation <- any(Rotation where (each.index = (myLandscape.rotationsList count (each != '') - 1)));
			hasRotationAssigned <- true;
			rotation.areaAssigned <- int(rotation.areaAssigned + size);
			do assign_initial_landuse(rotation);
			plannedLanduse <- myLandscape.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
		}

	}

	action setup_phenology
	{
	// Purpose: Define the phenology of each cover at the initialization
	// In : coversList, patch.plannedLanduse
	// Out: patch.currentLandcover
		ask patchesList
		{
			if plannedLanduse.isAnnual
			{
				currentLandcover <- myself.landcoversList first_with (each.name = "CoverBareGround"); // Bare ground for crop patches
				phenologicalStageIndex <- -1; // Default value of phenological stage
				if isInitialLanduse
				{ // If the crop is already available at the starting date
					currentLandcover <- any(Landcover where (each.name = "Cover" + plannedLanduse.name)); // Assign the landcover representing the crop		
					phenologicalStage <- '';
				}

				if plannedLanduse.isPhenologicallyDetailed and isInitialLanduse = false
				{ // If the crop is not available and phenology is detailed
					fieldSowingDelay <- rnd(plannedLanduse.cropSowingDelay); // Calculate the sowing delay
					fieldHarvestDelay <- rnd(plannedLanduse.cropHarvestDelay); // Calculate the harvest delay
					harvestDay <- -1; // Default value
				}

			}

		}

	}

	action update_landcovers
	{
	// Purpose: updates the landcover and landuse of each patch according to it's availability. 
	// In: patchesList, climate, landuses
	// InOut: patch.currentLandcover, patch.plannedLanduse, patch.randomSowingDelay, patch.successionIndex
		ask patchesList
		{
			if (plannedLanduse.isDynamicThroughTime)
			{ // If the landuse is temporally dynamic
				if plannedLanduse.isPartOfRotation
				{ // If the landuse is part of a rotation
					if plannedLanduse.isPhenologicallyDetailed and isInitialLanduse = false
					{
					// If the landuse is phenologically detailed and not the initial one
					// Calculate sowing date				
						int finalSowingDate <- plannedLanduse.startSowingDate + fieldSowingDelay;
						if finalSowingDate > 365
						{
							finalSowingDate <- finalSowingDate - 365;
						}
						// If sowing date reached and it is not raining then the field is sowed
						if julianDay = finalSowingDate and myself.climate.isRainingToday = false
						{
							currentLandcover <- myLandscape.landcoversList first_with (each.name = "CoverBareGround"); // Landcover is bare ground
							GDDCumulated <- 0.0; // Reset the cumulated degrees
							phenologicalStageIndex <- 0; // Reset the phenological stage
							phenologicalStage <- "Sowed"; // The field is sowed			
						}
						// If sowing date reached but it is raining, delay sowing to the next day					
						if julianDay = finalSowingDate and myself.climate.isRainingToday
						{
							fieldSowingDelay <- fieldSowingDelay + 1;
						}
						// If the crop is at any phenological stage further than sowed					
						if (phenologicalStageIndex >= 0)
						{
						// Update the growing degree days cumulated
							if (((myself.climate.minDayTemp + myself.climate.maxDayTemp) / 2) - plannedLanduse.baseTemperature) > 0
							{
								GDDCumulated <- GDDCumulated + (((myself.climate.minDayTemp + myself.climate.maxDayTemp) / 2) - plannedLanduse.baseTemperature);
							}
							// If the GDD value has reached the next crop stage threshold, then move on to the next phenological stage
							if phenologicalStageIndex < ((plannedLanduse.nbCropStages)) and GDDCumulated >= plannedLanduse.GDDThresholds[phenologicalStageIndex]
							{
								currentLandcover <- myLandscape.landcoversList first_with (each.name = "Cover" + plannedLanduse.name);
								phenologicalStage <- plannedLanduse.phenologicalStages[phenologicalStageIndex];
								// If the crop has reached the harvestable stage, then define the date of harvest
								if phenologicalStage = "Harvestable"
								{
									harvestDay <- julianDay + fieldHarvestDelay;
									if harvestDay > 365
									{
										harvestDay <- harvestDay - 365;
									}

								}

								phenologicalStageIndex <- phenologicalStageIndex + 1;
							}

						}
						// If harvest date reached and it is raining, delay harvest to the next day
						if (julianDay = harvestDay) and myself.climate.isRainingToday
						{
							harvestDay <- harvestDay + 1;
						}
						// If harvest date reached and it is not raining, harvest the crop and move on to the next landuse		
						if (julianDay = harvestDay) and myself.climate.isRainingToday = false
						{
							currentLandcover <- myself.landcoversList first_with (each.name = "CoverBareGround");
							phenologicalStage <- "";
							phenologicalStageIndex <- -1;
							successionIndex <- successionIndex + 1;
							if successionIndex > (length(rotation.succession)) - 1
							{
								successionIndex <- 0;
							}

							plannedLanduse <- myself.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
							if plannedLanduse.isPhenologicallyDetailed
							{
								fieldSowingDelay <- rnd(plannedLanduse.cropSowingDelay);
								fieldHarvestDelay <- rnd(plannedLanduse.cropHarvestDelay);
								harvestDay <- -1;
							}

						}
						// If the maximum harvest date reached and the crop has not been harvested yet, harvest it anyway
						if (julianDay = plannedLanduse.maximumHarvestDate and currentLandcover.name != "CoverBareGround")
						{
							currentLandcover <- myself.landcoversList first_with (each.name = "CoverBareGround");
							phenologicalStage <- "";
							phenologicalStageIndex <- -1;
							successionIndex <- successionIndex + 1;
							if successionIndex > (length(rotation.succession)) - 1
							{
								successionIndex <- 0;
							}

							plannedLanduse <- myself.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
							if plannedLanduse.isPhenologicallyDetailed
							{
								fieldSowingDelay <- rnd(plannedLanduse.cropSowingDelay);
								fieldHarvestDelay <- rnd(plannedLanduse.cropHarvestDelay);
								harvestDay <- -1;
							}

						}

					}
					// If the crop is not phenologically detailed, availability of the crop goes from sowing date to harvest date
else
					{
					// Crop is available at the sowing date	
						if julianDay = plannedLanduse.startSowingDate
						{
							currentLandcover <- myLandscape.landcoversList first_with (each.name = "Cover" + plannedLanduse.name);
							phenologicalStage <- "";
							phenologicalStageIndex <- -1;
						}
						// Crop is harvested at the harvest date	
						if julianDay = plannedLanduse.maximumHarvestDate and currentLandcover.name = "Cover" + plannedLanduse.name
						{
							currentLandcover <- myself.landcoversList first_with (each.name = "CoverBareGround");
							phenologicalStage <- "";
							phenologicalStageIndex <- -1;
							successionIndex <- successionIndex + 1;
							if successionIndex > (length(rotation.succession)) - 1
							{
								successionIndex <- 0;
							}

							plannedLanduse <- myself.landusesList first_with (each.name = string(rotation.succession[successionIndex]));
							isInitialLanduse <- false;
							if plannedLanduse.isPhenologicallyDetailed
							{
								fieldSowingDelay <- rnd(plannedLanduse.cropSowingDelay);
								fieldHarvestDelay <- rnd(plannedLanduse.cropHarvestDelay);
								harvestDay <- -1;
							}

						}

					}

				}

			}

		}

		ask patchesList
		{
		// Update the visual heights of all patches according to the crop and its phenological stage
			if currentLandcover.name = "Cover" + plannedLanduse.name and plannedLanduse.isPhenologicallyDetailed and isInitialLanduse = false
			{
				patchCoverHeight <- plannedLanduse.phenologicalHeight[phenologicalStageIndex - 1];
			} else
			{
				patchCoverHeight <- currentLandcover.baseHeight;
			}

		}

	}

}

species Climate
{
	float windSpeed; //Speed of the wind
	float windDirection; // Direction of the wind (360° degrees)
	int tempIndex; //Index in the temperature matrix
	int rainIndex; // Index in the rain matrix
	bool isRainingToday; //True if it has rained today ?
	float landscapeTemperature; // Temperature in the landscape (as data from the weather station)
	matrix<float> dayTemperatures <- 0.0 as_matrix { 24 / timestepHours, 1 }; // List of the temperatures of the day (every timestep)
	float meanDayTemp; //Mean temperature of the day
	float minDayTemp; //Min temperature of the day
	float maxDayTemp; //Max temperature of the day
	matrix dailyRainData; //Input csv as a matrix
	matrix windTempData; //Input csv as a matrix
	int tempIndMax; // Length of the climate matrix
	init
	{
	// Purpose: Initialize the climate attributes			
		dailyRainData <- matrix(rainCSV); //Matrix of the rainFile
		windTempData <- matrix(climateCSV); //Matrix of the climatFile (wind+temperatures)
		isRainingToday <- false; //Default value (it has not rained today)
		rainIndex <- julianDay+1; //Index in the rain matrix								
		tempIndex <- julianDay; //Index in the climate matrix
		tempIndMax <- ((windTempData column_at (0)) count (each != '') - 1); // Length of the climate Matrix				
	}

	action update_wind
	{
	// Purpose: Update the wind speed and direction at every step
	// In: windTempData
	// Out: windSpeed, windDirection			
		windSpeed <- float(windTempData[5, tempIndex]);
		windDirection <- float(windTempData[4, tempIndex]);
	}

	action update_temperature
	{
	// Purpose: Update the temperature at every step	
	// In: windTempData
	// Out: landscapeTemperature, meanDayTemp	
	// InOut: tempIndex																							
		landscapeTemperature <- float(windTempData[2, tempIndex]);
		int i <- int(hour / timestepHours);
		dayTemperatures[i, 0] <- (landscapeTemperature);
		if hour = 0
		{
			if hasHourlyWeatherData
			{ // If hourly data, calculate mean max and min temperatures
				meanDayTemp <- mean(dayTemperatures);
				minDayTemp <- min(dayTemperatures);
				maxDayTemp <- max(dayTemperatures);
			} else
			{ // If not hourly data, then follow the input order
				minDayTemp <- dayTemperatures[0, 0];
				maxDayTemp <- dayTemperatures[1, 0];
				meanDayTemp <- dayTemperatures[2, 0];
			}

		}

		tempIndex <- tempIndex + 1;
		if tempIndex > tempIndMax
		{
			tempIndex <- 0;
		}

	}

	action update_rain
	{
	// Purpose: Update the rain every day
	// In: dailyRainData
	// Out: rainedToday
	// InOut: rainInd	
		int raining <- int(dailyRainData[1, rainIndex]);
		if raining = 1
		{
			isRainingToday <- true;
		} else
		{
			isRainingToday <- false;
		}

		rainIndex <- rainIndex + 1;
		int rainIndMax <- ((dailyRainData column_at (0)) count (each != '') - 1);
		if rainIndex > rainIndMax
		{
			rainIndex <- 1;
		}

	}

}

species Patch
{
	string readRotation; // Rotation read in shapefile if info already in the input
	string readLanduse; // Cover read in the shapefile
	int iD; // Id of the patch
	float size; // Patch size in m²
	bool isInitialPatch; // Used to assign rotations (Reference patch or not)
	bool hasRotationAssigned; // Used to assign rotations (Has a rotation already been assigned to the patch)
	Rotation rotation; // Rotation assigned to the patch
	bool isInitialLanduse; // When we init, we have no info on DD so we use the dates for harvesting
	int successionIndex; // Index of the cover in the succession of the rotation	
	Landuse plannedLanduse; // Cover that will be grown this year
	Landcover currentLandcover; // Cover that is actually on the field		
	int fieldSowingDelay; // Delay due to human intervention for sowing
	int fieldHarvestDelay; // Delay due to human intervention for harvest		
	int harvestDay; // Day at which the patch will be harvested	
	float GDDCumulated; // Degree days absorbed by the plant
	int phenologicalStageIndex; // Index of the current phenological stage
	string phenologicalStage; // Name of the current phenological stage
	int patchCoverHeight; // Heigh of the current phenological stage
	action assign_initial_landuse (Rotation assignedRotation)
	{
	// Purpose : assing a starting point to the patch amongts all potentail starting points wihtin the rotation
		int k <- rnd(assignedRotation.duration);
		int successionPosition <- 0;
		if assignedRotation.firstDate > julianDay
		{
			k <- k + 1;
		}

		k <- julianDay + k * 365;
		loop x from: 0 to: assignedRotation.phenologyDates count (each != "") - 1
		{
			if assignedRotation.phenologyDates[x] < k
			{
				successionPosition <- successionPosition + 1;
			}

		}

		successionIndex <- int(successionPosition / 2);
		if successionPosition = (assignedRotation.phenologyDates count (each != ""))
		{
			successionIndex <- 0;
		}

		if successionPosition mod 2 = 0
		{
			isInitialLanduse <- false;
		} else
		{
			isInitialLanduse <- true;
		}

	}

	aspect base
	{
		draw shape color: currentLandcover.color depth: patchCoverHeight border: # black;
	}

}

species Rotation
{
	int duration; // Number of years covered by the rotation
	list<Landuse> succession; //The succession of covers in the rotation
	float referenceArea; //Area that has to be assigned to the rotation
	int index; //Index of the rotation in world.myLandscape.rotationsList
	// (used when creating the attribution matrix)
	int firstDate; // Used when defining starting points       
	int areaAssigned; // Used when assigning rotations (Current area assigned to the rotation)
	list<int> phenologyDates; // Used when defining starting points
}

species Landcover
{
	rgb color; // Color of representation
	string abbreviation; // 3 Letter abreviation
	int baseHeight; // Height of the cover in a 3D map
	int index; // Index of the cover in Landscape.coversList 
	// (used when creating the attribution matrix)		                        

}

species Landuse
{
	string abbreviation; // 3 Letter abreviation
	int index; // Index of the cover in Landscape.coversList 
	// (used when creating the attribution matrix)  
	bool isDynamicThroughTime; // Static or dynamic
	bool isPartOfRotation; // Will this landuse vary according to a rotation assigned to the patch
	bool isPhenologicallyDetailed; // Shoud a detailed phenology be considered for this landuse	                                    
	bool isAnnual; // Annual or perennial species
	bool isClustered; //Is the crop clustered in the landscape
	int startSowingDate; //Date when it is possible to sow
	int maximumHarvestDate; //Maximum date to harvest
	int cropHarvestDelay; // Harvest possible delay in days (time window) 
	int cropSowingDelay; // Sowing possible delay in day (time window)
	list<string> phenologicalStages; // Name of each phenological stage
	list<int> GDDThresholds; // GDD Thresholds for each phenological stage
	list<int> phenologicalHeight; // Height of each phenological stage
	int nbCropStages; // Number of crop stages
	int baseTemperature; // Base growth temperature of the crop								

}

experiment Simulate_Landscape type: gui{
// Purpose: Simulation with step by step visual outputs

	parameter 'Select the initiailization file defining the simulation' var: PARAMETERS_FILE category: 'File input';
			
	output {
		display Landscape_2D refresh: every(1) {	// Shows a 2D image of the landscape
			species Patch aspect: base ;					
		}
		display Landscape_3D type:opengl {				// Show a 3D representation of the landscape
			 species Patch aspect: base;
		}
	   display Climat refresh: every(1)  {
			chart "Mean Temperature" type: series background: rgb("white") size: {1,0.4} position: {0, 0.05}  {	
				// Mean temperature of the landscape 
				data "Temperature" value: world.myLandscape.climate.landscapeTemperature   color: rgb("blue") ;
			}
			chart "Wind Speed" type: series background: rgb("white") size: {1,0.4} position: {0, 1000}  {	
				// Mean windspeed of the landscape
				data "WindSpeed" value: world.myLandscape.climate.windSpeed color: rgb("blue") ;
			}
		}	
		display "Daily Crop Surfaces" refresh: every(1) {
			chart "Daily Crop Surfaces" type: series background: #white {//x_serie_labels:list(julianDay) {
				data "Wheat" value: areaWheat color: rgb("yellow") line_visible: false;
				data "Corn" value: areaCorn color: rgb("orange") line_visible: false;
				data "Sorghum" value: areaSorghum color: rgb("red")line_visible: false  ;	
				data "Rapeseed" value: areaRapeseed color: rgb("pink")line_visible: false ;
				data "Temporarypasture" value: areaTemporarypasture color: rgb("green")line_visible: false ;
				data "Sunflower" value: areaSunflower color: rgb("magenta")line_visible: false ;						
			}
		}
 
		display "Wheat Crop stages" refresh: every(1)  {
			chart "Daily surface of each Wheat Crop stage" type: series background: #white {//x_serie_labels:list(julianDay) {
				data"Sowed" value: areaWheatSowed color:rgb(0,205,0);
				data"Emergence" value: areaWheatEmerg color:rgb(34,139,34);				
				data"Boot" value: areaWheatBoot color:rgb(154,205,50);				
				data"Head" value: areaWheatHead color:rgb(255,255,0);				
				data"Flowering" value: areaWheatFlow color:rgb(255,165,0);	
				data"Maturity" value: areaWheatMaturity color:rgb(255,127,0);				
				data"Harvestable" value: areaWheatHarvest color:rgb(205,51,51);																										
			}
		}
				
		monitor julianDay value: julianDay refresh: every(1)  ;
		monitor Days_elapsed value:daysElapsed refresh: every(1) ;
		monitor Hour value: hour refresh: every(1)  ;
		monitor Year value: year refresh: every(1) ;

	}	
}

experiment Batch_Simulation type: batch repeat: 30 keep_seed: false  until: (year > 11){
// Purpose: Possibility to launch batch simulations, not updating the graphical outputs and thus gaining time

}
